/* 
 * This plugin does 22 diffrent sweeps from one strip to
 * another, (If one of the strips doesn't exist it goes to black)
 * Created by mein@cs.umn.edu 	Oct 25 1999
 */

#include "plugin.h"

enum {DO_LEFT_RIGHT, DO_RIGHT_LEFT, DO_DOWN_UP, DO_UP_DOWN, 
      DO_LOWER_LEFT_UPPER_RIGHT, DO_UPPER_RIGHT_LOWER_LEFT,
      DO_UPPER_LEFT_LOWER_RIGHT, DO_LOWER_RIGHT_UPPER_LEFT, 
      DO_HORZ_OUT, DO_HORZ_IN, DO_VERT_OUT, DO_VERT_IN,
      DO_HORZ_VERT_OUT, DO_HORZ_VERT_IN, DO_LEFT_DOWN_RIGHT_UP_OUT,
      DO_LEFT_DOWN_RIGHT_UP_IN, DO_LEFT_UP_RIGHT_DOWN_OUT, 
      DO_LEFT_UP_RIGHT_DOWN_IN, DO_DIAG_OUT, DO_DIAG_IN, DO_DIAG_OUT_2,
      DO_DIAG_IN_2};

/* ******************** GLOBAL VARIABLES ***************** */


char name[24]= "Sweep";	

/* structure for buttons, 
 *  butcode      name           default  min  max  0
 */

VarStruct varstr[]= {
   LABEL,	"In: 1|2 strips", 0.0, 0.0, 0.0, "", 
   NUM|INT,	"Type",	    0.0,	0.0,	21.0, "Our type of Sweep", 
   NUM|INT,	"Start Frame", 1,	0,	500.0, "Starting Frame",
   NUM|INT, 	"Steps",     20.0,	0.0,	500.0, 
	"Size of a single interval",
   TOG|INT,	"Animated", 0.0,	0.0,	1.0, "For (Ipo) animated blur", 
};

/* The cast struct is for input in the main doit function
   Varstr and Cast must have the same variables in the same order */ 

typedef struct Cast {
	int dummy;			/* because of the 'label' button */
	int sweeptype;
	int frame0;
        int steps;
	float use_ipo;
} Cast;


/* cfra: the current frame */

float cfra;

void plugin_seq_doit(Cast *, float, float, int, int, 
   ImBuf *, ImBuf *, ImBuf *, ImBuf *);

/* ******************** Fixed functions ***************** */

int plugin_seq_getversion(void) {
	return B_PLUGIN_VERSION;
}

void plugin_but_changed(int but) {
}

void plugin_init() {
}

void plugin_getinfo(PluginInfo *info) {
	info->name= name;
	info->nvars= sizeof(varstr)/sizeof(VarStruct);
	info->cfra= &cfra;

	info->varstr= varstr;

	info->init= plugin_init;
	info->seq_doit= (SeqDoit) plugin_seq_doit;
	info->callback= plugin_but_changed;
}

int check_zone(int x, int y, int xo, int yo, Cast *cast) {
   int pos = cast->steps * (cfra - cast->frame0);
   float halfx = xo/2;
   float halfy = yo/2;

   switch (cast->sweeptype) {
      case DO_LEFT_RIGHT:
         if (x > pos) return 1;
         return 0;
         break;
      case DO_RIGHT_LEFT:
         if (x < (xo - pos)) return 1;
         return 0;
	 break;
      case DO_DOWN_UP:
         if (y > pos ) return 1;
         return 0;
	 break;
      case DO_UP_DOWN:
         if (y < (yo - pos)) return 1;
         return 0;
	 break;
      case DO_LOWER_LEFT_UPPER_RIGHT:
         if ((x + yo - y) < pos) return 0;
         return 1;
	 break;
      case DO_UPPER_RIGHT_LOWER_LEFT:
         if ((xo - x + y) < pos) return 0;
         return 1;
	 break;
      case DO_UPPER_LEFT_LOWER_RIGHT:
         if ((x + y) < pos ) return 0;
         return 1;
	 break;
      case DO_LOWER_RIGHT_UPPER_LEFT:
         if ((xo - x + yo - y) < pos) return 0;
         return 1;
	 break;
      case DO_HORZ_OUT:
         if ((x < (halfx - pos)) || (x > (halfx + pos))) return 1;
         return 0;
	 break;
      case DO_HORZ_IN:
         if ((x >pos) && (x < (xo - pos))) return 1;
         return 0;
	 break;
      case DO_VERT_OUT:
         if ((y < (halfy - pos)) || (y > (halfy + pos))) return 1;
         return 0;
	 break;
      case DO_VERT_IN:
         if ((y >pos) && (y < (yo - pos))) return 1;
         return 0;
	 break;
      case DO_HORZ_VERT_OUT:
         if (((x > (halfx - pos)) && (x < (halfx + pos))) || 
             ((y > (halfy - pos)) && (y < (halfy + pos)))) return 0;
         return 1;
	 break;
      case DO_HORZ_VERT_IN:
         if ((x <pos) || ((xo - x) <  pos) || (y < pos) || 
	     ((yo - y) < pos)) return 1;
         return 0;
	 break;
      case DO_LEFT_DOWN_RIGHT_UP_OUT:
         if (((x - halfx + y - halfy) < pos) && 
             ((halfx -x + halfy -y ) < pos)) return 1;
         return 0;
	 break;
      case DO_LEFT_DOWN_RIGHT_UP_IN:
         if (((y + xo - x) < pos) || ((yo - y + x) < pos)) return 1;
         return 0;
	 break;
      case DO_LEFT_UP_RIGHT_DOWN_OUT:
         if (((x - halfx + yo - y - halfy) < pos) && 
             ((halfx - x + halfy - yo + y ) < pos)) return 1;
         return 0;
	 break;
      case DO_LEFT_UP_RIGHT_DOWN_IN:
         if (((x+ y) < pos) || ((xo -x + yo - y) < pos )) return 1;
         return 0;
	 break;
      case DO_DIAG_OUT:
         if ((((x - halfx + y - halfy) < pos) && 
             ((halfx -x + halfy -y ) < pos)) && 
             (((x - halfx + yo - y - halfy) < pos)) && 
             ((halfx -x + halfy -yo + y) < pos)) return 1;
         return 0;
	 break;
      case DO_DIAG_IN:
         if ((x + y) < pos || ((xo -x + yo - y) < pos) ||
             ((x + yo -y) < pos) || ((y + xo - x) < pos)) return 1;
         return 0;
 	 break;
      case DO_DIAG_OUT_2:
         if ((((x - halfx + y - halfy) < pos) &&
             ((halfx -x + halfy -y ) < pos)) ||
             (((x - halfx + yo - y - halfy) < pos)) &&
             ((halfx -x + halfy -yo + y) < pos)) return 1;
         return 0;
         break;
       case DO_DIAG_IN_2:
         if ((((y + xo - x) < pos) || ((yo - y + x) < pos)) &&
         (((x+ y) < pos) || ((xo -x + yo - y) < pos ))) return 1;
         return 0;
         break;
   }
   return 0;
}

void plugin_seq_doit(Cast *cast, float facf0, float facf1, int xo, int yo, 
	ImBuf *ibuf1, ImBuf *ibuf2, ImBuf *outbuf, ImBuf *use) {
   int x;
   int y;

   char *in1=(char *)ibuf1->rect;
   char *in2=(char *)ibuf2->rect;
   char *out=(char *)outbuf->rect;
   for(y=0;y<yo;y++) {
      for(x=0;x<xo;x++) {
         if (check_zone(x,y,xo,yo,cast)) {
            if (in1 != NULL) {
               out[0] = in1[0];
               out[1] = in1[1];
               out[2] = in1[2];
               out[3] = in1[3];
            } else {
               out[0] = 255;
               out[1] = 0;
               out[2] = 0;
               out[3] = 0;
            }
         } else {
            if (in2 != NULL) {
               out[0] = in2[0];
               out[1] = in2[1];
               out[2] = in2[2];
               out[3] = in2[3];
            } else {
               out[0] = 255;
               out[1] = 0;
               out[2] = 0;
               out[3] = 0;
            }
         }
         out+=4;
         if (in1 !=NULL) {
           in1+=4;
         }
         if (in2 !=NULL) {
            in2+=4;
         }
      }
   }
}

